/*
 * ad9850.h
 *
 *  Created on: Oct 26, 2015
 *      Author: NMR
 */

#ifndef AD9850_H_
#define AD9850_H_

/*
 * main.c
 *
 *  Created on: Sep 20, 2015
 *      Author: NMR
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>

#define DDS_CLK 125000000UL

#define PIN_WCLK  PD0
#define PIN_DATA  PD1
#define PIN_FQUP  PD2
#define PIN_RESET PD3

void InitAD9850(void) //TODO: set port and port direction!!
{
	DDRD  = 0x0F;
	PORTD = 0x00;

	PORTD &= ~(1<<PIN_RESET);
	PORTD &= ~(1<<PIN_WCLK);
	PORTD &= ~(1<<PIN_FQUP);
	PORTD &= ~(1<<PIN_DATA);
}

//reset sequence is:
  // CLOCK & LOAD = LOW
  //  Pulse RESET high for a few uS (use 5 uS here)
  //  Pulse CLOCK high for a few uS (use 5 uS here)
  //  Set DATA to ZERO and pulse LOAD for a few uS (use 5 uS here)

  // data sheet diagrams show only RESET and CLOCK being used to reset the device, but I see no output unless I also
  // toggle the LOAD line here.
//BZW.
// Set CLK and FQUP to LOW
// Create High Pulse to RESET
// Create HIGH Pulse to CLK
// Set SERIALDATA to ZERO and finally create a HIGH pulse to FQUP
void ddsReset(void)
{
	// Reset sequence following datasheet instructions

	//1. Set WCLK and FQUP to LOW
	PORTD &= ~(1<<PIN_WCLK);
	PORTD &= ~(1<<PIN_FQUP);

	//2. Generate high pulse on RESET
	PORTD &= ~(1<<PIN_RESET);
	PORTD |= (1<<PIN_RESET);
	PORTD &= ~(1<<PIN_RESET);

	//3. Generate high pulse on WCLK
	PORTD &= ~(1<<PIN_WCLK);
	PORTD |= (1<<PIN_WCLK);
	PORTD &= ~(1<<PIN_WCLK);

	//4. Set DATA to zero.
	PORTD &= ~(1<<PIN_DATA);

	//5. Generate high pulse on FQUP
	PORTD &= ~(1<<PIN_FQUP);
	PORTD |= (1<<PIN_FQUP);
	PORTD &= ~(1<<PIN_FQUP);
}

void ddsSetFrequency(uint32_t frequency)
{
	uint64_t tuningword;
	uint8_t msg[5];// = {0x2D, 0x3E, 0x05, 0x00, 0x00}; //ca. 10kHz
	uint8_t tmp;
	int i, j;

	tuningword = (frequency * 4294967296) / DDS_CLK;

	msg[0] = (uint8_t)(tuningword & 0xFF);
	msg[1] = (uint8_t)( (tuningword>>8 ) & 0xFF);
	msg[2] = (uint8_t)( (tuningword>>16 ) & 0xFF);
	msg[3] = (uint8_t)( (tuningword>>24 ) & 0xFF);
	msg[4] = 0x00;

	for(j=0; j<5; j++)
	{
		for(i=0; i<8; i++)
		{
			tmp = (msg[j] & (1<<i));

			PORTD &= ~(1<<PIN_WCLK);

			if (tmp > 0)
				PORTD |= ((tmp >> i) << PIN_DATA);
			else
				PORTD &= ~(1<<PIN_DATA);

			PORTD |= (1<<PIN_WCLK);
		}
	}
	PORTD &= ~(1<<PIN_WCLK);
	PORTD |= (1<<PIN_FQUP);
	PORTD &= ~(1<<PIN_FQUP);
}

#endif /* AD9850_H_ */
