#include <avr/io.h>
#include <avr/interrupt.h>

#include "button.h"
#include "display.h"
#include "ad9850.h"

#define BUTTON_OK    BUTTON1
#define BUTTON_RIGHT BUTTON2
#define BUTTON_LEFT  BUTTON3
#define BUTTON_DOWN  BUTTON4
#define BUTTON_UP    BUTTON5
#define BUTTON_ALL   (1<<BUTTON_OK) | (1<<BUTTON_UP) | (1<<BUTTON_DOWN) | (1<<BUTTON_LEFT) | (1<<BUTTON_RIGHT)

#ifndef TRUE
#define TRUE         1
#endif
#ifndef FALSE
#define FALSE        0
#endif

volatile uint8_t position = 0;
volatile uint8_t digit[8] = {0,0,0,0,0,0,0,0};

uint32_t GetFreqency(uint8_t values[])
{
  uint32_t retval = 0;
  uint8_t i;

  for( i = 0; i < 8; i++ )
    retval = retval * 10 + values[7-i];

  return retval;
}
/*
uint32_t GetFreqency(uint8_t values[])
{
	uint32_t retval;

	retval  = values[0];
	retval += values[1] * 10;
	retval += values[2] * 100;
	retval += values[3] * 1000;
	retval += values[4] * 10000;
	retval += values[5] * 100000;
	retval += values[6] * 1000000;
	retval += values[7] * 10000000;

	return retval;
}*/


ISR(TIMER1_COMPA_vect)
{
	static uint8_t flag;

	SendDigits(digit);

	if(flag++ & 0x01)
	{
		Send( ((position + 1) << 8) | CODE_BLANK );
	}
	else
	{
		Send( ((position + 1) << 8) | digit[position] );
	}
}

void MoveLeft(void)
{
	position = (position + 1) % 8;
}

void MoveRight(void)
{
	if(position > 0)
	{
		position = (position - 1);
	}
	else
	{
		position = 7;
	}
}

void IncreaseDigit(void)
{
	digit[position] = (digit[position] + 1) % 10;
}

void DecreaseDigit(void)
{
	if(digit[position] > 0)
	{
		digit[position] = (digit[position] - 1);
	}
	else
	{
		digit[position] = 9;
	}
}

int main(void)
{
	uint8_t inputMode;
	uint32_t frequency;

	DDRD = 0x00;
	PORTD = 0x00;

	InitAD9850();
	InitButton();
	InitDisplay();

	TCCR1B |= ((1<<WGM12) | (1<<CS10) | (1<<CS11));

	OCR1AH = 0xF4;
	OCR1AL = 0x24;

	TIMSK |= (1<<OCIE1A);

	sei();

	SendDigits(digit);

	ddsReset();
	inputMode = TRUE;

	while(1)
	{
		if(inputMode)
		{
			if( buttonChanged( 1<<BUTTON_LEFT) )
			{
				MoveLeft();
			}

			if( buttonChanged( 1<<BUTTON_RIGHT) )
			{
				MoveRight();
			}

			if( buttonChanged( 1<<BUTTON_UP) )
			{
				IncreaseDigit();
			}

			if( buttonChanged( 1<<BUTTON_DOWN) )
			{
				DecreaseDigit();
			}

			if( buttonChanged( 1<<BUTTON_OK ) )
			{
				TIMSK &= ~(1<<OCIE1A);
				SendDigits(digit);
				inputMode = FALSE;
				frequency = GetFreqency(digit);
				ddsSetFrequency(frequency);
			}
		}
		else
		{
			if( buttonChanged( BUTTON_ALL ) )
			{
				TIMSK |= (1<<OCIE1A);
				SendDigits(digit);
				inputMode = TRUE;
			}
		}
	}

	return 0;
}
